unit Mesh;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, dglOpenGL;

type

//  TVec3 = array[0..2] of single;

  { TMesh }

  TMesh = class(TObject)
  private
    Fcol1: TGLVectorf3;
    Fcol2: TGLVectorf3;
    FSektoren: cardinal;
    procedure SetSektoren(AValue: cardinal);
  public
    property col1: TGLVectorf3 write Fcol1;
    property col2: TGLVectorf3 write Fcol2;
    property Sektoren: cardinal read FSektoren write SetSektoren;
    constructor Create;
    procedure Disk(ri, ra: single);
    procedure Kreissegment(r, w: single);
    procedure Arc(ri, ra, wa, we: single);
  end;

function vec3(v0, v1, v2: GLfloat): TGLVectorf3;

implementation

function vec3(v0, v1, v2: GLfloat): TGLVectorf3;
begin
  Result[0] := v0;
  Result[1] := v1;
  Result[2] := v2;
end;


{ TMesh }

procedure TMesh.SetSektoren(AValue: cardinal);
begin
  if FSektoren = AValue then begin
    Exit;
  end;
  FSektoren := AValue;
end;

constructor TMesh.Create;
begin
  Fcol1 := vec3(1.0, 1.0, 1.0);
  Fcol2 := vec3(0.0, 0.0, 0.0);
  Sektoren := 36;
end;

procedure TMesh.Disk(ri, ra: single);
var
  w, seg: single;
  i: integer;
begin
  glbegin(GL_TRIANGLE_STRIP);
  seg := 360 / FSektoren;
  for i := 0 to FSektoren do begin
    w := i * seg / 180 * Pi;
    glColor3fv(@Fcol1);
    glVertex3f(cos(w) * ri, sin(w) * ri, 0);
    glColor3fv(@Fcol2);
    glVertex3f(cos(w) * ra, sin(w) * ra, 0);
  end;
  glEnd();
end;

procedure TMesh.Arc(ri, ra, wa, we: single);

  procedure CalcAndDraw(w: single);
  var
    c, s: single;
  begin
    c := cos(w / 180 * Pi);
    s := sin(w / 180 * Pi);
    glColor3fv(@Fcol1);
    glVertex3f(c * ri, s * ri, 0);
    glColor3fv(@Fcol2);
    glVertex3f(c * ra, s * ra, 0);
  end;

begin
  glBegin(GL_TRIANGLE_STRIP);
  repeat
    CalcAndDraw(wa);
    wa += 360 / FSektoren;
  until wa > we;
  CalcAndDraw(we);
  glEnd();
end;

procedure TMesh.Kreissegment(r, w: single);

  procedure CalcAndDraw(w: single);
  begin
    w := w / 180 * Pi;
    glVertex3f(cos(w) * r, sin(w) * r, 0);
  end;

var
  w2: single;

begin
  w2 := 0;

  glBegin(GL_POLYGON);
  glColor3fv(@Fcol1);
  glVertex3f(r, 0, 0);
  glColor3fv(@Fcol2);
  repeat
    w2 += 10;
    CalcAndDraw(w2);
  until w2 > w;
  w2 := w;
  CalcAndDraw(w2);
  glEnd();
end;


end.
