unit Rad;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, dglOpenGL, Mesh;

type

  { TRad }

  TRad = class(TObject)
  private
    FSpeichen: cardinal;
    Radius: single;
    Mesh: TMesh;
    ListIndex: GLuint;
  public
    constructor Create(ARadius: single; Speichen: cardinal = 15);
    destructor Destroy; override;
    procedure Draw(startwinkel: single);
  end;

implementation

{ TRad }

constructor TRad.Create(ARadius: single; Speichen: cardinal);
const
  segWinkelGegengewicht = 90.0;
var
  i: integer;
  l: single;
begin
  inherited Create;
  Mesh := TMesh.Create;
  Speichen := 15;
  Radius := ARadius;
  ListIndex := glGenLists(1);

  glNewList(ListIndex, GL_COMPILE);

  l := Radius / 20;
  for i := 0 to speichen - 1 do begin
    glRotatef(360 / speichen, 0, 0, 1);

    glBegin(GL_QUADS);
    glColor3f(0.5, 0.1, 0.1);
    glVertex3f(-l, Radius - l, 0);
    glVertex3f(-l, 0, 0);
    glColor3f(0.7, 0.2, 0.2);
    glVertex3f(l, 0, 0);
    glVertex3f(l, Radius - l, 0);
    glEnd();
  end;

  glTranslatef(0, 0, -0.01); // Nabe/Spurkranz vor Speichen
  Mesh.col1 := vec3(0.1, 0.0, 0.0);
  Mesh.col2 := vec3(0.9, 0.0, 0.1);
  Mesh.Disk(Radius / 1.15, Radius);

  Mesh.col1 := vec3(0.1, 0.0, 0.0);
  Mesh.col2 := vec3(0.9, 0.0, 0.1);
  Mesh.Disk(0.0, Radius / 4);

  Mesh.col1 := vec3(0.7, 0.7, 0.7);
  Mesh.col2 := vec3(0.5, 0.5, 0.5);
  Mesh.Disk(Radius, Radius * 1.05);

  glTranslatef(0, 0, -0.01); // Schraube vor Nabe
  Mesh.col1 := vec3(0.0, 0.0, 0.0);
  Mesh.col2 := vec3(0.2, 0.2, 0.2);
  Mesh.Sektoren := 6;
  Mesh.Disk(0.0, Radius / 10);
  Mesh.Sektoren := 36;

  glRotatef(-segWinkelGegengewicht / 2 + 180, 0, 0, 1); // Gegengewicht
  Mesh.col1 := vec3(0.8, 0.3, 0.3);
  Mesh.col2 := vec3(0.4, 0.2, 0.2);
  Mesh.Kreissegment(Radius / 1.05, segWinkelGegengewicht);

  glEndList;
end;

destructor TRad.Destroy;
begin
  Mesh.Free;
  glDeleteLists(ListIndex, 1);
  inherited Destroy;
end;

procedure TRad.Draw(startwinkel: single);
begin
  glPushMatrix;
  glRotatef(startwinkel, 0, 0, 1);
  glCallList(ListIndex);
  glPopMatrix;
end;


end.
